@@ -10,36 +10,51 @@
 #include "iwyu_globals.h"
 
 #include <algorithm>                    // for sort, make_pair
+#include <climits>
 #include <cstdio>                       // for printf
 #include <cstdlib>                      // for atoi, exit, getenv
+#include <cstring>
 #include <map>                          // for map
 #include <set>                          // for set
 #include <string>                       // for string, operator<, etc
 #include <utility>                      // for make_pair, pair
 
+#include "clang/AST/PrettyPrinter.h"
+#include "clang/Basic/DirectoryEntry.h"
+#include "clang/Basic/LangOptions.h"
+#include "clang/Basic/Version.h"
+#include "clang/Driver/ToolChain.h"
+#include "clang/Frontend/CompilerInstance.h"
+#include "clang/Lex/DirectoryLookup.h"
+#include "clang/Lex/HeaderSearch.h"
+#include "clang/Lex/Preprocessor.h"
 #include "iwyu_cache.h"
-#include "iwyu_include_picker.h"
 #include "iwyu_getopt.h"
+#include "iwyu_include_picker.h"
 #include "iwyu_lexer_utils.h"
 #include "iwyu_location_util.h"
 #include "iwyu_path_util.h"
 #include "iwyu_port.h"  // for CHECK_, etc
 #include "iwyu_regex.h"
-#include "iwyu_stl_util.h"
 #include "iwyu_string_util.h"
 #include "iwyu_verrs.h"
 #include "iwyu_version.h"
-#include "llvm/Support/raw_ostream.h"
-#include "clang/AST/PrettyPrinter.h"
-#include "clang/Basic/FileManager.h"
-#include "clang/Basic/Version.h"
-#include "clang/Driver/ToolChain.h"
-#include "clang/Frontend/CompilerInstance.h"
-#include "clang/Lex/HeaderSearch.h"
-#include "clang/Lex/Preprocessor.h"
+#include "llvm/ADT/StringRef.h"
+#include "llvm/Option/ArgList.h"
 
-using clang::driver::ToolChain;
+// TODO: Clean out pragmas as IWYU improves.
+// IWYU pragma: no_include <unistd.h>
+// IWYU pragma: no_include "llvm/ADT/iterator.h"
+
+using clang::CompilerInstance;
+using clang::HeaderSearch;
+using clang::LangOptions;
 using clang::OptionalDirectoryEntryRef;
+using clang::OptionalFileEntryRef;
+using clang::PrintingPolicy;
+using clang::SourceManager;
+using clang::driver::ToolChain;
+using clang::getClangFullVersion;
 using std::make_pair;
 using std::map;
 using std::string;
@@ -48,12 +63,10 @@ using std::vector;
 namespace include_what_you_use {
 
 static CommandlineFlags* commandline_flags = nullptr;
-static clang::SourceManager* source_manager = nullptr;
-static ToolChain::CXXStdlibType cxx_stdlib_type =
-    ToolChain::CXXStdlibType::CST_Libstdcxx;
+static SourceManager* source_manager = nullptr;
 static IncludePicker* include_picker = nullptr;
-static const clang::LangOptions default_lang_options;
-static const clang::PrintingPolicy default_print_policy(default_lang_options);
+static const LangOptions default_lang_options;
+static const PrintingPolicy default_print_policy(default_lang_options);
 static SourceManagerCharacterDataGetter* data_getter = nullptr;
 static FullUseCache* function_calls_full_use_cache = nullptr;
 static FullUseCache* class_members_full_use_cache = nullptr;
@@ -137,8 +150,7 @@ static void PrintVersion() {
   if (!iwyu_rev.empty()) {
     llvm::outs() << " (git:" << iwyu_rev << ")";
   }
-  llvm::outs() << " based on " << clang::getClangFullVersion()
-               << "\n";
+  llvm::outs() << " based on " << getClangFullVersion() << "\n";
 }
 
 static bool ParseIntegerOptarg(const char* optarg, int* res) {
@@ -411,7 +423,7 @@ static vector<HeaderSearchPath> NormalizeHeaderSearchPaths(
 // Asks clang what the search-paths are for include files, normalizes
 // them, and returns them in a vector.
 static vector<HeaderSearchPath> ComputeHeaderSearchPaths(
-    clang::HeaderSearch* header_search) {
+    HeaderSearch* header_search) {
   map<string, HeaderSearchPath::Type> search_path_map;
   for (auto it = header_search->system_dir_begin();
        it != header_search->system_dir_end(); ++it) {
@@ -433,7 +445,7 @@ static vector<HeaderSearchPath> ComputeHeaderSearchPaths(
   return NormalizeHeaderSearchPaths(search_path_map);
 }
 
-static CStdLib DeriveCStdLib(clang::CompilerInstance&) {
+static CStdLib DeriveCStdLib() {
   if (GlobalFlags().no_default_mappings)
     return CStdLib::None;
   if (GlobalFlags().HasExperimentalFlag("clang_mappings"))
@@ -441,25 +453,25 @@ static CStdLib DeriveCStdLib(clang::CompilerInstance&) {
   return CStdLib::Glibc;
 }
 
-static CXXStdLib DeriveCXXStdLib(clang::CompilerInstance& compiler) {
+static CXXStdLib DeriveCXXStdLib(const ToolChain& toolchain) {
   if (GlobalFlags().no_default_mappings)
     return CXXStdLib::None;
   if (GlobalFlags().HasExperimentalFlag("clang_mappings"))
     return CXXStdLib::ClangSymbols;
-  if (cxx_stdlib_type == ToolChain::CXXStdlibType::CST_Libcxx)
-    return CXXStdLib::Libcxx;
-  return CXXStdLib::Libstdcxx;
-}
 
-void ParseToolChain(const clang::driver::ToolChain& tc) {
-  // Get standard library that has been requested. Get this from the
-  // ToolChain. This should already have been parsed, so pass in an
-  // empty arglist.
+  // Get standard library requested for the compilation. ToolChain caches the
+  // already-parsed args, so pass in an empty arglist.
   llvm::opt::InputArgList nullargs;
-  cxx_stdlib_type = tc.GetCXXStdlibType(nullargs);
+  switch (toolchain.GetCXXStdlibType(nullargs)) {
+    case ToolChain::CXXStdlibType::CST_Libcxx:
+      return CXXStdLib::Libcxx;
+    case ToolChain::CXXStdlibType::CST_Libstdcxx:
+      return CXXStdLib::Libstdcxx;
+  }
+  CHECK_UNREACHABLE_("covered switch for CXXStdlibType above");
 }
 
-void InitGlobals(clang::CompilerInstance& compiler) {
+void InitGlobals(CompilerInstance& compiler, const ToolChain& toolchain) {
   source_manager = &compiler.getSourceManager();
   data_getter = new SourceManagerCharacterDataGetter(*source_manager);
   vector<HeaderSearchPath> search_paths = ComputeHeaderSearchPaths(
@@ -467,8 +479,8 @@ void InitGlobals(clang::CompilerInstance& compiler) {
   SetHeaderSearchPaths(search_paths);
 
   RegexDialect regex_dialect = GlobalFlags().regex_dialect;
-  CStdLib cstdlib = DeriveCStdLib(compiler);
-  CXXStdLib cxxstdlib = DeriveCXXStdLib(compiler);
+  CStdLib cstdlib = DeriveCStdLib();
+  CXXStdLib cxxstdlib = DeriveCXXStdLib(toolchain);
   include_picker = new IncludePicker(regex_dialect, cstdlib, cxxstdlib);
 
   function_calls_full_use_cache = new FullUseCache;
@@ -497,7 +509,7 @@ CommandlineFlags* MutableGlobalFlagsForTesting() {
   return commandline_flags;
 }
 
-clang::SourceManager* GlobalSourceManager() {
+SourceManager* GlobalSourceManager() {
   CHECK_(source_manager && "Must call InitGlobals() before calling this");
   return source_manager;
 }
@@ -512,7 +524,7 @@ IncludePicker* MutableGlobalIncludePicker() {
   return include_picker;
 }
 
-const clang::PrintingPolicy& DefaultPrintPolicy() {
+const PrintingPolicy& DefaultPrintPolicy() {
   return default_print_policy;
 }
 
@@ -534,7 +546,7 @@ void AddGlobToReportIWYUViolationsFor(const string& glob) {
   commandline_flags->check_also.insert(NormalizeFilePath(glob));
 }
 
-bool ShouldReportIWYUViolationsFor(const clang::FileEntry* file) {
+bool ShouldReportIWYUViolationsFor(OptionalFileEntryRef file) {
   const string filepath = GetFilePath(file);
   for (const string& glob : GlobalFlags().check_also)
     if (GlobMatchesPath(glob.c_str(), filepath.c_str()))
@@ -547,7 +559,7 @@ void AddGlobToKeepIncludes(const string& glob) {
   commandline_flags->keep.insert(NormalizeFilePath(glob));
 }
 
-bool ShouldKeepIncludeFor(const clang::FileEntry* file) {
+bool ShouldKeepIncludeFor(OptionalFileEntryRef file) {
   if (GlobalFlags().keep.empty())
     return false;
   const string filepath = GetFilePath(file);
