@@ -15,12 +15,13 @@
 #include <string>                       // for string
 #include <vector>                       // for vector
 
+#include "clang/Basic/FileEntry.h"
+
 namespace clang {
 class CompilerInstance;
-class FileEntry;
-class HeaderSearch;
 class SourceManager;
 struct PrintingPolicy;
+
 namespace driver {
 class ToolChain;
 }  // namespace driver
@@ -32,10 +33,10 @@ using std::set;
 using std::string;
 using std::vector;
 
-enum class RegexDialect;
 class FullUseCache;
 class IncludePicker;
 class SourceManagerCharacterDataGetter;
+enum class RegexDialect;
 
 // To set up the global state you need to parse options with OptionsParser when
 // main starts and to call InitGlobals after the clang infrastructure is set up.
@@ -59,8 +60,8 @@ class OptionsParser {
   const char** clang_argv_;
 };
 
-void ParseToolChain(const clang::driver::ToolChain&);
-void InitGlobals(clang::CompilerInstance& compiler);
+void InitGlobals(clang::CompilerInstance& compiler,
+                 const clang::driver::ToolChain& toolchain);
 
 // Can be called by tests -- doesn't need a SourceManager or
 // argc/argv.  Note that GlobalSourceManager() and DefaultDataGetter()
@@ -137,12 +138,12 @@ FullUseCache* ClassMembersFullUseCache();
 // They are specified as glob file-patterns (which behave just as they
 // do in the shell).  TODO(csilvers): use a prefix instead? allow '...'?
 void AddGlobToReportIWYUViolationsFor(const string& glob);
-bool ShouldReportIWYUViolationsFor(const clang::FileEntry* file);
+bool ShouldReportIWYUViolationsFor(clang::OptionalFileEntryRef file);
 
 // For the commandline option --keep.
 // Similar to AddGlobToReportIWYUViolationsFor.
 void AddGlobToKeepIncludes(const string& glob);
-bool ShouldKeepIncludeFor(const clang::FileEntry* file);
+bool ShouldKeepIncludeFor(clang::OptionalFileEntryRef file);
 
 }  // namespace include_what_you_use
 
