@@ -25,8 +25,10 @@
 // Clang uses the type FileEntry to identify a physical file in the
 // file system.  A FileEntry is created for each source file Clang
 // processes.  Clang never creates two FileEntry objects for the same
-// file.  Therefore we use const FileEntry* in IWYU as unique IDs for
-// files.
+// file, so FileEntry objects have pointer identity. Clang wraps
+// FileEntry in a couple of stronger types with similar semantics
+// (FileEntryRef and OptionalFileEntryRef). We use these wrappers
+// as file identities in IWYU.
 //
 // Clang's FileID type is a misnomer.  It's actually an ID of a
 // particular #include statement.  If a file is #included in two
@@ -40,14 +42,16 @@
 #ifndef INCLUDE_WHAT_YOU_USE_IWYU_LOCATION_UTIL_H_
 #define INCLUDE_WHAT_YOU_USE_IWYU_LOCATION_UTIL_H_
 
+#include <optional>
 #include <string>                       // for string
 
-#include "iwyu_globals.h"
-#include "iwyu_path_util.h"
-#include "clang/Basic/FileManager.h"
+#include "clang/Basic/FileEntry.h"
 #include "clang/Basic/SourceLocation.h"
 #include "clang/Basic/SourceManager.h"
 #include "clang/Lex/Token.h"
+#include "iwyu_globals.h"
+#include "iwyu_path_util.h"
+#include "llvm/ADT/StringRef.h"
 
 namespace clang {
 class Decl;
@@ -64,21 +68,11 @@ using std::string;
 //------------------------------------------------------------
 // Helper functions for FileEntry.
 
-// Some symbols are directly defined by the compiler.  For them, the
-// definition location points to the "<built-in>" file.
-inline bool IsBuiltinFile(const clang::FileEntry* file) {
-  return file == nullptr;
-}
-
-// There are two kinds of symbols that are not defined in the source
-// files: the compiler can define some standard symbols
-// (e.g. __FILE__), and the user can define macros on the command line
-// of the compiler using -D.  A symbol appears to be defined in file
-// "<built-in>" in the first case, and "<command line>" in the second.
-// IsBuiltinOrCommandLineFile(file) returns true if it's either of the
-// two cases.
-inline bool IsBuiltinOrCommandLineFile(const clang::FileEntry* file) {
-  return IsBuiltinFile(file) || file->getName().equals("<command line>");
+inline const clang::FileEntry* RawFileEntry(clang::OptionalFileEntryRef file) {
+  if (!file) {
+    return nullptr;
+  }
+  return &file->getFileEntry();
 }
 
 // When macro args are concatenated e.g. '#define CAT(A, B) A##B', their
@@ -87,9 +81,13 @@ inline bool IsBuiltinOrCommandLineFile(const clang::FileEntry* file) {
 // This function returns true if the provided loc is in scratch space.
 bool IsInScratchSpace(clang::SourceLocation loc);
 
-inline string GetFilePath(const clang::FileEntry* file) {
-  return (IsBuiltinFile(file) ? "<built-in>" :
-          NormalizeFilePath(file->getName().str()));
+// Resolve canonical file path from various file entry types.
+inline string GetFilePath(clang::OptionalFileEntryRef file) {
+  return (!file ? "<built-in>" : NormalizeFilePath(file->getName().str()));
+}
+
+inline string GetFilePath(clang::FileEntryRef file) {
+  return NormalizeFilePath(file.getName().str());
 }
 
 //------------------------------------------------------------
@@ -115,35 +113,35 @@ inline bool IsInMacro(clang::SourceLocation loc) {
 inline int GetLineNumber(clang::SourceLocation loc) {
   if (!loc.isValid())
     return -1;
-  const clang::FullSourceLoc fullloc(loc, *GlobalSourceManager());
+  clang::SourceManager& sm = *GlobalSourceManager();
   bool invalid = false;
-  int retval = fullloc.getSpellingLineNumber(&invalid);
+  int retval = sm.getSpellingLineNumber(loc, &invalid);
   if (invalid)
-    retval = fullloc.getExpansionLineNumber(&invalid);
+    retval = sm.getExpansionLineNumber(loc, &invalid);
   if (invalid)
     retval = -1;
   return retval;
 }
 
 // The rest of this section of the file is for returning the
-// FileEntry* corresponding to a source location: the file that the
+// FileEntry corresponding to a source location: the file that the
 // location is in.  This is a surprising amount of work.
 
 // Tells which #include loc comes from.
 // This is the most basic FileEntry getter, it only does a simple lookup in
 // SourceManager to determine which file the location is associated with.
-inline const clang::FileEntry* GetLocFileEntry(clang::SourceLocation loc) {
+inline clang::OptionalFileEntryRef GetLocFileEntry(clang::SourceLocation loc) {
   // clang uses the name FileID to mean 'a filename that was reached via
   // a particular series of #includes.'  (What one might think a FileID
   // might be -- a unique reference to a filesystem object -- is
-  // actually a FileEntry*.)
+  // actually a FileEntry.)
   const clang::SourceManager& source_manager = *GlobalSourceManager();
-  return source_manager.getFileEntryForID(source_manager.getFileID(loc));
+  return source_manager.getFileEntryRefForID(source_manager.getFileID(loc));
 }
 
-inline const clang::FileEntry* GetFileEntry(clang::SourceLocation loc) {
+inline clang::OptionalFileEntryRef GetFileEntry(clang::SourceLocation loc) {
   if (!loc.isValid())
-    return nullptr;
+    return std::nullopt;
 
   // We want where the user actually writes the token, instead of
   // where it appears as part of a macro expansion.  For example, in:
@@ -153,7 +151,7 @@ inline const clang::FileEntry* GetFileEntry(clang::SourceLocation loc) {
   //
   // FOO(z) will expand to 'z + y', where symbol z's location is
   // foo.h, line 5, and its spelling location is bar.cc, line 10.
-  const clang::FileEntry* retval = GetLocFileEntry(GetSpellingLoc(loc));
+  clang::OptionalFileEntryRef retval = GetLocFileEntry(GetSpellingLoc(loc));
 
   // Sometimes the spelling location is NULL, because the symbol is
   // 'spelled' via macro concatenation.  For instance, all the
@@ -183,12 +181,15 @@ clang::SourceLocation GetLocation(const clang::NestedNameSpecifierLoc* nnsloc);
 clang::SourceLocation GetLocation(const clang::TemplateArgumentLoc* argloc);
 
 // These define default implementations of GetFileEntry() and
-// GetPath() in terms of GetLocation().  As long as an object defines
+// GetFilePath() in terms of GetLocation().  As long as an object defines
 // its own GetLocation(), it will get these other two for free.
-template<typename T> const clang::FileEntry* GetFileEntry(const T& obj) {
+template <typename T>
+clang::OptionalFileEntryRef GetFileEntry(const T& obj) {
   return GetFileEntry(GetLocation(obj));
 }
-template<typename T> const string GetFilePath(const T& obj) {
+
+template <typename T>
+const string GetFilePath(const T& obj) {
   return GetFilePath(GetFileEntry(obj));
 }
 
@@ -201,8 +202,8 @@ template<typename T> const string GetFilePath(const T& obj) {
 // the other looking at the output of cc -E or equivalent.
 template<typename T, typename U>
 inline bool IsBeforeInTranslationUnit(const T& a, const U& b) {
-  const clang::FullSourceLoc a_loc(GetLocation(a), *GlobalSourceManager());
-  const clang::FullSourceLoc b_loc(GetLocation(b), *GlobalSourceManager());
+  clang::SourceLocation a_loc = GetLocation(a);
+  clang::SourceLocation b_loc = GetLocation(b);
   // Inside a macro, everything has the same instantiation location.
   // We'd like to use spelling-location to break that tie, but it's
   // unreliable since a or b might be spelled in "<scratch space>".
@@ -216,7 +217,7 @@ inline bool IsBeforeInTranslationUnit(const T& a, const U& b) {
   if ((IsInMacro(a_loc) || IsInMacro(b_loc)) &&
       GetInstantiationLoc(a_loc) == GetInstantiationLoc(b_loc))
     return true;
-  return a_loc.isBeforeInTranslationUnitThan(b_loc);
+  return GlobalSourceManager()->isBeforeInTranslationUnit(a_loc, b_loc);
 }
 
 // Like IsBeforeInTranslationUnit, but both a and b must be
@@ -228,9 +229,29 @@ inline bool IsBeforeInSameFile(const T& a, const U& b) {
   return IsBeforeInTranslationUnit(a, b);
 }
 
+// Returns true if argument is one of the special files used by Clang for
+// implicit buffers ("<built-in>", "<command-line>", etc).
+// A null value is considered the same as "<built-in>".
+inline bool IsSpecialFile(clang::OptionalFileEntryRef file) {
+  if (!file)
+    return true;
+  return IsSpecialFilename(file->getName());
+}
+
+// Returns true if obj is in a special file, as defined above.
+// Note that it also returns true for objects at invalid locations, as they
+// resolve to a null file.
+template <typename T>
+inline bool IsInSpecialFile(const T& obj) {
+  return IsSpecialFile(GetFileEntry(obj));
+}
+
 // Returns true if the given declaration is located in a header file.
 bool IsInHeader(const clang::Decl*);
 
+// Returns true if file is a system header.
+bool IsSystemHeader(clang::OptionalFileEntryRef file);
+
 }  // namespace include_what_you_use
 
 #endif  // INCLUDE_WHAT_YOU_USE_IWYU_LOCATION_UTIL_H_
