@@ -9,16 +9,13 @@
 
 #include "iwyu_path_util.h"
 
-#include <algorithm>                    // for std::replace
-#include <cstddef>
 #include <cstring>                      // for strlen
 #include <system_error>
 
-#include "iwyu_stl_util.h"
-
+#include "iwyu_port.h"
+#include "iwyu_string_util.h"
 #include "llvm/ADT/SmallString.h"
 #include "llvm/ADT/StringRef.h"
-#include "llvm/ADT/STLExtras.h"
 #include "llvm/Support/FileSystem.h"
 #include "llvm/Support/Path.h"
 
@@ -84,11 +81,13 @@ string Basename(const string& path) {
 }
 
 string GetCanonicalName(string file_path) {
-  // For this special 'path' we just return it.
-  // Note that we leave the 'quotes' to make it different from regular paths.
-  if (file_path == "<built-in>" || file_path == "<stdin>")
+  // Clang special filenames are already canonical.
+  // <stdin> is not a special filename, but it's canonical too.
+  if (IsSpecialFilename(file_path) || file_path == "<stdin>")
     return file_path;
 
+  // All known special filenames which look like quoted-includes are handled
+  // above. Reject anything else that looks like a quoted-include.
   CHECK_(!IsQuotedInclude(file_path));
 
   file_path = NormalizeFilePath(file_path);
@@ -128,13 +127,8 @@ string GetCanonicalName(string file_path) {
 string NormalizeFilePath(const string& path) {
   llvm::SmallString<128> normalized(path);
   llvm::sys::path::remove_dots(normalized, /*remove_dot_dot=*/true);
-
-#ifdef _WIN32
   // Canonicalize directory separators (forward slashes considered canonical.)
-  std::replace(normalized.begin(), normalized.end(), '\\', '/');
-#endif
-
-  return normalized.str().str();
+  return llvm::sys::path::convert_to_slash(normalized);
 }
 
 string NormalizeDirPath(const string& path) {
@@ -198,10 +192,7 @@ string ConvertToQuotedInclude(const string& filepath,
     // quoted include by just stripping the prefix.
 
     if (StripPathPrefix(&path, entry.path)) {
-      if (entry.path_type == HeaderSearchPath::kSystemPath)
-        return "<" + path + ">";
-      else
-        return "\"" + path + "\"";
+      return AddQuotes(path, entry.path_type == HeaderSearchPath::kSystemPath);
     }
   }
 
@@ -209,7 +200,7 @@ string ConvertToQuotedInclude(const string& filepath,
   // Uses the implicit "-I <basename current file>" entry on the search path.
   if (!includer_path.empty())
     StripPathPrefix(&path, NormalizeDirPath(includer_path));
-  return "\"" + path + "\"";
+  return AddQuotes(path, /*angled=*/false);
 }
 
 bool IsQuotedInclude(const string& s) {
@@ -225,4 +216,11 @@ bool IsSystemIncludeFile(const string& filepath) {
   return ConvertToQuotedInclude(filepath)[0] == '<';
 }
 
+string AddQuotes(string include_name, bool angled) {
+  if (angled) {
+      return "<" + include_name + ">";
+  }
+  return "\"" + include_name + "\"";
+}
+
 }  // namespace include_what_you_use
