@@ -15,27 +15,34 @@
 __author__ = 'wan@google.com (Zhanyong Wan)'
 
 import difflib
+import functools
 import operator
 import os
 import re
 import shlex
+import shutil
 import subprocess
 import sys
 import unittest
 
 # These are the warning/error lines that iwyu.cc produces when --verbose >= 3
-_EXPECTED_DIAGNOSTICS_RE = re.compile(r'^\s*//\s*IWYU:\s*(.*)$')
+_EXPECTED_DIAGNOSTICS_RE = re.compile(r'^\s*// IWYU:\s*(.*)$')
 _ACTUAL_DIAGNOSTICS_RE = re.compile(r'^(.*?):(\d+):\d+:\s*'
                                     r'(?:warning|error|fatal error):\s*(.*)$')
 
+# Diagnostics without line/column number (e.g. from driver), e.g.:
+# // IWYU~: argument unused during compilation
+_EXPECTED_NOLOC_DIAGS_RE = re.compile(r'^\s*// IWYU~: (.*)$')
+_ACTUAL_NOLOC_DIAGS_RE = re.compile(r'^(?:warning|error|fatal error):\s*(.*)$')
+
 # This is the final summary output that iwyu.cc produces when --verbose >= 1
 # The summary for a given source file should appear in that source file,
 # surrounded by '/**** IWYU_SUMMARY' and '***** IWYU_SUMMARY */'.
 # The leading summary line may also have an expected exit-code in parentheses
 # after the summary marker: '/**** IWYU_SUMMARY(10)'.
-_EXPECTED_SUMMARY_START_RE = re.compile(r'/\*+\s*IWYU_SUMMARY')
-_EXPECTED_SUMMARY_EXIT_CODE_RE = re.compile(r'/\*+\s*IWYU_SUMMARY\((\d+)\)')
-_EXPECTED_SUMMARY_END_RE = re.compile(r'\**\s*IWYU_SUMMARY\s*\*+/')
+_EXPECTED_SUMMARY_START_RE = re.compile(r'/\*+ IWYU_SUMMARY')
+_EXPECTED_SUMMARY_EXIT_CODE_RE = re.compile(r'/\*+ IWYU_SUMMARY\((\d+)\)')
+_EXPECTED_SUMMARY_END_RE = re.compile(r'\** IWYU_SUMMARY \*+/')
 _ACTUAL_SUMMARY_START_RE = re.compile(r'^(.*?) should add these lines:$')
 _ACTUAL_SUMMARY_END_RE = re.compile(r'^---$')
 _ACTUAL_REMOVAL_LIST_START_RE = re.compile(r'.* should remove these lines:$')
@@ -44,7 +51,7 @@ _NODIFFS_RE = re.compile(r'^\((.*?) has correct #includes/fwd-decls\)$')
 # This is an IWYU_ARGS line that specifies launch arguments for a test in its
 # source file. Example:
 # // IWYU_ARGS: -Xiwyu --mapping_file=... -I .
-_IWYU_TEST_RUN_ARGS_RE = re.compile(r'^//\sIWYU_ARGS:\s(.*)$')
+_IWYU_TEST_RUN_ARGS_RE = re.compile(r'^// IWYU_ARGS:\s(.*)$')
 
 # Text matching a condition, either as part of IWYU_REQUIRES,
 # IWYU_UNSUPPORTED or IWYU_XFAIL.
@@ -66,6 +73,14 @@ _IWYU_TEST_PREREQ_RE = re.compile(r'^// IWYU_(REQUIRES|UNSUPPORTED): ' +
 _IWYU_TEST_XFAIL_RE = re.compile(r'^// IWYU_XFAIL(?:: ' +
                                  _IWYU_CONDITION + r')?$')
 
+
+def cache(fn):
+  """ functools has a cache decorator starting with Python 3.9. Support 3.2+ with
+  a local alias.
+  """
+  return functools.lru_cache(maxsize=None)(fn)
+
+
 class Features:
   """ Implements feature support functions for prerequisites and
       expected failures. """
@@ -73,12 +88,54 @@ class Features:
   # Parses the output of 'include-what-you-use -print-targets'.
   SUPPORTED_TARGET_RE = re.compile(r'^\s+([a-z0-9_-]+)\s* - .*$')
 
+  # Parses the output of 'include-what-you-use -dM -E'.
+  PREPROCESSOR_MACRO_RE = re.compile(r'^#define ([A-z_][A-z0-9_]*)')
+
   def __init__(self):
     # Maps all known features names to prerequisite predicates.
     self.known = {
-      'has-target': Features.HasTarget
+      'has-target': Features.HasTarget,
+      'uses-cstdlib': Features.UsesCstdlib,
+      'uses-cxxstdlib': Features.UsesCXXstdlib,
     }
 
+  @staticmethod
+  def _GetPreprocessorMacros(lang, header):
+    iwyu = _GetIwyuPath()
+    src = '#include %s' % header
+
+    cmd = [iwyu, '-dM', '-E', '-x', lang]
+    # Take into account extra arguments specified in IWYU_EXTRA_ARGS -
+    # the user may be trying to test with different includes, standard
+    # libraries or whatever.
+    #
+    # For feature tests we _don't_ take into account test-specific
+    # IWYU_ARGS.
+    #  * the argument could be the thing that breaks things
+    #  * the feature result should not be test specific
+    cmd += _GetExtraArgs()
+    cmd += ['-']
+
+    result = subprocess.run(cmd, capture_output=True, input=src, text=True)
+
+    macros = []
+    for line in result.stdout.splitlines():
+      m = Features.PREPROCESSOR_MACRO_RE.match(line)
+      if m:
+        macros.append(m.group(1))
+
+    return macros
+
+  @staticmethod
+  @cache
+  def _GetCXXPreprocessorMacros():
+    return Features._GetPreprocessorMacros('c++', '<cstdlib>')
+
+  @staticmethod
+  @cache
+  def _GetCPreprocessorMacros():
+    return Features._GetPreprocessorMacros('c', '<stdlib.h>')
+
   @staticmethod
   def HasTarget(target):
     iwyu = _GetIwyuPath()
@@ -92,42 +149,49 @@ class Features:
         supported.append(m.group(1))
     return target in supported
 
+  @staticmethod
+  def UsesCXXstdlib(lib):
+    id_macros = {
+      'libcxx': ['_LIBCPP_VERSION'],
+      'libstdcxx': ['__GLIBCXX__'],
+      # https://github.com/microsoft/STL/wiki/Macro-_MSVC_STL_UPDATE
+      'msvc': ['_MSVC_STL_UPDATE'],
+    }
+    macros = id_macros.get(lib)
+    if macros is None:
+      raise SyntaxError('Unknown C++ standard library: %s' % lib)
+
+    return all(macro in Features._GetCXXPreprocessorMacros() for macro in macros)
+
+  @staticmethod
+  def UsesCstdlib(lib):
+    id_macros = {
+      'glibc': ['__GLIBC__'],
+      'llvmlibc': ['__LLVM_LIBC__'],
+
+      # On MacOS sys/cdefs.h defines a bunch of macros. Use
+      # __DARWIN_C_LEVEL to identify the MacOS C library.
+      # https://opensource.apple.com/source/xnu/xnu-7195.81.3/bsd/sys/cdefs.h.auto.html
+      'macos': ['__DARWIN_C_LEVEL'],
+
+      # _MSC_VER identifies the Microsoft compiler rather than the
+      # standard library (MSVCRT or UCRT). clang will also define this
+      # for MS targets.
+      'msvcrt': ['_MSC_VER'],
+      'newlib': ['__NEWLIB__'],
+      'uclibc': ['__UCLIBC__'],
+    }
+    macros = id_macros.get(lib)
+    if macros is None:
+      raise SyntaxError('Unknown C standard library: %s' % lib)
+
+    # TODO: This needs refining to ensure it correctly identifies said libraries
+    return all(macro in Features._GetCPreprocessorMacros() for macro in macros)
 
 _FEATURES = Features()
 
 
-def _Which(program, paths):
-    """Searches specified paths for program."""
-    if sys.platform == 'win32' and not program.lower().endswith('.exe'):
-        program += '.exe'
-
-    for path in paths:
-        candidate = os.path.join(os.path.normpath(path), program)
-        if os.path.isfile(candidate):
-            return candidate
-
-    return None
-
-
-_IWYU_PATH = None
-_SYSTEM_PATHS = [p.strip('"') for p in os.environ["PATH"].split(os.pathsep)]
-_IWYU_PATHS = [
-    '../../../../Debug+Asserts/bin',
-    '../../../../Release+Asserts/bin',
-    '../../../../Release/bin',
-    '../../../../build/Debug+Asserts/bin',
-    '../../../../build/Release+Asserts/bin',
-    '../../../../build/Release/bin',
-    # Linux/Mac OS X default out-of-tree paths.
-    '../../../../../build/Debug+Asserts/bin',
-    '../../../../../build/Release+Asserts/bin',
-    '../../../../../build/Release/bin',
-    # Windows default out-of-tree paths.
-    '../../../../../build/bin/Debug',
-    '../../../../../build/bin/Release',
-    '../../../../../build/bin/MinSizeRel',
-    '../../../../../build/bin/RelWithDebInfo',
-    ]
+_IWYU_PATH = shutil.which('include-what-you-use')
 
 
 def SetIwyuPath(iwyu_path):
@@ -139,15 +203,8 @@ def SetIwyuPath(iwyu_path):
 
 def _GetIwyuPath():
   """Returns the path to IWYU or raises IOError if it cannot be found."""
-  global _IWYU_PATH
-
   if not _IWYU_PATH:
-    iwyu_paths = _IWYU_PATHS + _SYSTEM_PATHS
-    _IWYU_PATH = _Which('include-what-you-use', iwyu_paths)
-    if not _IWYU_PATH:
-      raise IOError('Failed to locate IWYU.\nSearched\n %s' %
-                    '\n '.join(iwyu_paths))
-
+    raise IOError('\'include-what-you-use\' not found in PATH')
   return _IWYU_PATH
 
 
@@ -161,6 +218,14 @@ def _GetCommandOutput(command):
   return p.returncode, lines
 
 
+def _GetExtraArgs():
+  """Retrieve IWYU_EXTRA_ARGS from the environment and return as a list"""
+  env_iwyu_extra_args = os.getenv('IWYU_EXTRA_ARGS')
+  if env_iwyu_extra_args:
+    return shlex.split(env_iwyu_extra_args)
+  return []
+
+
 def _GetMatchingLines(regex, file_names):
   """Returns a map: file location => string matching `regex`.
 
@@ -223,6 +288,27 @@ def _GetActualDiagnostics(actual_output):
   return actual_diagnostics
 
 
+def _GetExpectedNoLocDiagnosticRegexes(filename):
+  """Returns a list of regexes for the file."""
+  expected = []
+  with open(filename, 'r') as fileobj:
+    for line in fileobj:
+      m = _EXPECTED_NOLOC_DIAGS_RE.match(line.strip())
+      if m:
+        expected.append(re.compile(m.group(1)))
+  return expected
+
+
+def _GetActualNoLocDiagnostics(actual_output):
+  """Returns a list of diagnostic messages."""
+  actual_diagnostics = []
+  for line in actual_output:
+    m = _ACTUAL_NOLOC_DIAGS_RE.match(line.strip())
+    if m:
+      actual_diagnostics.append(m.group(1))
+  return actual_diagnostics
+
+
 def _StripCommentFromLine(line):
   """Removes the "// ..." comment at the end of the given line."""
   return re.sub(r'\s*//.*$', '', line)
@@ -368,9 +454,8 @@ def _GetActualSummaries(output):
   return actual_summaries
 
 
-def _VerifyDiagnosticsAtLoc(loc_str, regexes, diagnostics):
-  """Verify the diagnostics at the given location; return a list of failures."""
-
+def _VerifyDiagnostics(regexes, diagnostics, loc_str=''):
+  """Verify the diagnostics; return a list of failures."""
   # Find out which regexes match a diagnostic and vice versa.
   matching_regexes = [[] for unused_i in range(len(diagnostics))]
   matched_diagnostics = [[] for unused_i in range(len(regexes))]
@@ -406,7 +491,7 @@ def _VerifyDiagnosticsAtLoc(loc_str, regexes, diagnostics):
           % (regexes[r_index].pattern,
              '\n'.join([diagnostics[d_index] for d_index in d_indexes])))
 
-  return ['%s %s' % (loc_str, message) for message in failure_messages]
+  return ['\n%s%s' % (loc_str, message) for message in failure_messages]
 
 
 def _CompareExpectedAndActualDiagnostics(expected_diagnostic_regexes,
@@ -419,11 +504,18 @@ def _CompareExpectedAndActualDiagnostics(expected_diagnostic_regexes,
     # Find all regexes and actual diagnostics for the given location.
     regexes = expected_diagnostic_regexes.get(loc, [])
     diagnostics = actual_diagnostics.get(loc, [])
-    failures += _VerifyDiagnosticsAtLoc('\n%s:%s:' % loc, regexes, diagnostics)
+    failures += _VerifyDiagnostics(regexes, diagnostics, '%s:%s: ' % loc)
 
   return failures
 
 
+def _CompareExpectedAndActualNoLocDiagnostics(expected_regexes,
+                                              actual_diagnostics):
+  """Verify that every regex in expected matches a line in actual; return a list
+  of failures"""
+  return _VerifyDiagnostics(expected_regexes, actual_diagnostics)
+
+
 def _CompareExpectedAndActualSummaries(expected_summaries, actual_summaries):
   """Verify that the summaries are as expected; return a list of failures."""
 
@@ -568,9 +660,7 @@ def TestIwyuOnRelativeFile(cc_file, cpp_files_to_check, verbose=False):
   env_verbose_level = os.getenv('IWYU_VERBOSE')
   if env_verbose_level:
     cmd += ['-Xiwyu', '--verbose=' + env_verbose_level]
-  env_iwyu_extra_args = os.getenv('IWYU_EXTRA_ARGS')
-  if env_iwyu_extra_args:
-      cmd += shlex.split(env_iwyu_extra_args)
+  cmd += _GetExtraArgs()
   cmd += [cc_file]
 
   if verbose:
@@ -585,9 +675,15 @@ def TestIwyuOnRelativeFile(cc_file, cpp_files_to_check, verbose=False):
     raise AssertionError('Unexpected exit code, wanted %d, was %d' %
                          (expected_exit_code, exit_code))
 
+  # Check diagnostics without location (from driver)
+  failures = _CompareExpectedAndActualNoLocDiagnostics(
+      _GetExpectedNoLocDiagnosticRegexes(cc_file),
+      _GetActualNoLocDiagnostics(output))
+
+  # Check IWYU diagnostics
   expected_diagnostics = _GetMatchingLines(
       _EXPECTED_DIAGNOSTICS_RE, cpp_files_to_check)
-  failures = _CompareExpectedAndActualDiagnostics(
+  failures += _CompareExpectedAndActualDiagnostics(
       _GetExpectedDiagnosticRegexes(expected_diagnostics),
       _GetActualDiagnostics(output))
 
