@@ -178,6 +178,42 @@ class IWYUToolTests(unittest.TestCase):
             invocation.will_block(random.random() / 100)
         self.assertEqual(self._execute(invocations, jobs=100), 6)
 
+    def test_worst_returncode(self):
+        # If no invocations fail, we return zero
+        invocations = [MockInvocation() for _ in range(4)]
+        exit_codes = [0, 0, 0, 0]
+        for invocation, exit_code in zip(invocations, exit_codes):
+            invocation.will_returncode(exit_code)
+        self.assertEqual(self._execute(invocations), 0)
+
+        # If one invocation fails, its exit code is returned
+        invocations = [MockInvocation() for _ in range(4)]
+        exit_codes = [0, 1, 0, 0]
+        for invocation, exit_code in zip(invocations, exit_codes):
+            invocation.will_returncode(exit_code)
+        self.assertEqual(self._execute(invocations), 1)
+
+        # The highest of all positive exit codes is returned
+        invocations = [MockInvocation() for _ in range(4)]
+        exit_codes = [0, 1, 2, 3]
+        for invocation, exit_code in zip(invocations, exit_codes):
+            invocation.will_returncode(exit_code)
+        self.assertEqual(self._execute(invocations), 3)
+
+        # The lowest of all negative exit codes is returned
+        invocations = [MockInvocation() for _ in range(4)]
+        exit_codes = [-9, -10, -15, -1]
+        for invocation, exit_code in zip(invocations, exit_codes):
+            invocation.will_returncode(exit_code)
+        self.assertEqual(self._execute(invocations), -15)
+
+        # Negative take precedence over positive/zero
+        invocations = [MockInvocation() for _ in range(4)]
+        exit_codes = [0, 1, -1, 1]
+        for invocation, exit_code in zip(invocations, exit_codes):
+            invocation.will_returncode(exit_code)
+        self.assertEqual(self._execute(invocations), -1)
+
     @unittest.skipIf(sys.platform.startswith('win'), "POSIX only")
     def test_is_subpath_of_posix(self):
         self.assertTrue(iwyu_tool.is_subpath_of('/a/b/c.c', '/a/b'))
@@ -327,6 +363,35 @@ class BootstrapTests(unittest.TestCase):
         self.assertEqual(['arg1', '--', 'another_arg1'],
                          self.main.call_args['extra_args'])
 
+    def test_jobs_normal(self):
+        """ Nonzero values for -j are forwarded as they are. """
+        argv = ['iwyu_tool.py', '-p', '.', '-j', '4']
+        iwyu_tool._bootstrap(argv)
+        self.assertEqual(4, self.main.call_args['jobs'])
+
+
+    def test_jobs_zero(self):
+        """ -j=0 is translated to a nonzero job count. """
+        argv = ['iwyu_tool.py', '-p', '.', '-j', '0']
+        iwyu_tool._bootstrap(argv)
+        self.assertIsNotNone(self.main.call_args['jobs'])
+        self.assertLess(0, self.main.call_args['jobs'],
+                        'The automatically chosen job count should be >0.')
+
+    def test_jobs_novalue(self):
+        """ It is allowed to use -j without a value. """
+        argv = ['iwyu_tool.py', '-j', '-p', '.']
+        iwyu_tool._bootstrap(argv)
+        self.assertIsNotNone(self.main.call_args['jobs'])
+        self.assertLess(0, self.main.call_args['jobs'],
+                        'The automatically chosen job count should be >0.')
+
+    def test_jobs_default(self):
+        """ By default, we run singlethreaded. """
+        argv = ['iwyu_tool.py', '-p', '.']
+        iwyu_tool._bootstrap(argv)
+        self.assertEqual(1, self.main.call_args['jobs'])
+
 
 class CompilationDBTests(unittest.TestCase):
     def setUp(self):
@@ -379,5 +444,20 @@ class CompilationDBTests(unittest.TestCase):
             [iwyu_tool.IWYU_EXECUTABLE, '-c', 'test.c'])
 
 
+class FindIWYUTests(unittest.TestCase):
+    def test_iwyu_binary_made_absolute(self):
+        oldval = os.environ.get('IWYU_BINARY')
+        try:
+            # Seed IWYU_BINARY with a relative path.
+            os.environ['IWYU_BINARY'] = './build/include-what-you-use'
+
+            # Check that find_iwyu returns absolute.
+            path = iwyu_tool.find_include_what_you_use()
+            self.assertTrue(os.path.isabs(path), 'Expected absolute: %r' % path)
+        finally:
+            if oldval:
+                os.environ['IWYU_BINARY'] = oldval
+
+
 if __name__ == '__main__':
     unittest.main()
