@@ -9,13 +9,15 @@
 
 #include "iwyu_verrs.h"
 
+#include <string>
+
 #include "iwyu_globals.h"
 #include "iwyu_location_util.h"
 #include "iwyu_path_util.h"
 
 namespace include_what_you_use {
 
-using clang::FileEntry;
+using clang::OptionalFileEntryRef;
 
 namespace {
 int verbose_level = 1;
@@ -29,7 +31,7 @@ int GetVerboseLevel() {
   return verbose_level;
 }
 
-bool ShouldPrintSymbolFromFile(const FileEntry* file) {
+bool ShouldPrintSymbolFromFile(OptionalFileEntryRef file) {
   if (GetVerboseLevel() < 5) {
     return false;
   } else if (GetVerboseLevel() < 10) {
