@@ -28,6 +28,14 @@ int UseStruct(struct Struct* s);
 struct ForwardDeclared;
 void UseForwardDeclared(struct ForwardDeclared*);
 
+// If a forward-declaration is seen before an actual struct declaration in the
+// same file, no diagnostic is expected (see issue #1065).
+typedef struct local_struct local_struct_t;
+struct local_struct {
+  int x;
+  local_struct_t* next;
+};
+
 /**** IWYU_SUMMARY
 
 tests/c/elaborated_struct.c should add these lines:
