@@ -14,8 +14,13 @@
 
 class Base;
 
+namespace ns {
+using ::Base;
+}
+
 class ReturnsBase {
   virtual Base* non_covariant() = 0;
+  virtual ns::Base* non_covariant_sugared() = 0;
 
   virtual Base* covariant_derived() = 0;
   virtual const Class* covariant_cv_qual() = 0;
@@ -28,6 +33,12 @@ class ReturnsDerived : public ReturnsBase {
     return 0;
   }
 
+  static Base b;
+
+  decltype(b)* non_covariant_sugared() {
+    return 0;
+  }
+
   // C++ [class.virtual]p7, second bullet
   // Trigger covariant return types:
   //  Base is an unambiguous and accessible direct or indirect base class of
