@@ -10,7 +10,7 @@
 #ifndef INCLUDE_WHAT_YOU_USE_TESTS_CXX_EXPLICIT_INSTANTIATION2_TEMPLATE_DIRECT_H_
 #define INCLUDE_WHAT_YOU_USE_TESTS_CXX_EXPLICIT_INSTANTIATION2_TEMPLATE_DIRECT_H_
 
-#include "explicit_instantiation2-template_bool.h"
-#include "explicit_instantiation2-template_short.h"
+#include "tests/cxx/explicit_instantiation2-template_bool.h"
+#include "tests/cxx/explicit_instantiation2-template_short.h"
 
 #endif  // INCLUDE_WHAT_YOU_USE_TESTS_CXX_EXPLICIT_INSTANTIATION2-TEMPLATE_DIRECT_H_
