@@ -37,6 +37,14 @@ void FunctionProtoClassArguments() {
   // IWYU: IndirectClass is...*indirect.h
   FunctionStruct<IndirectClass(char)> f2;
   (void)f2;
+
+  // IWYU: IndirectClass is...*indirect.h
+  using ProvidingFunctionAlias = IndirectClass(IndirectClass);
+  FunctionStruct<ProvidingFunctionAlias> f3;
+  // IWYU: IndirectClass is...*indirect.h
+  FunctionStruct<NonProvidingFunctionAlias1> f4;
+  // IWYU: IndirectClass is...*indirect.h
+  FunctionStruct<NonProvidingFunctionAlias2> f5;
 }
 
 // ---------------------------------------------------------------
@@ -89,6 +97,11 @@ struct StaticTemplateFieldStruct {
   static NonProvidingAlias<5> aliasedNonProviding;
 };
 
+template <typename T>
+void FnUsingNested() {
+  (void)sizeof(Outer<Inner<T>>);
+}
+
 void NestedTemplateArguments() {
   // IWYU: IndirectClass needs a declaration
   // IWYU: IndirectClass is...*indirect.h
@@ -103,6 +116,10 @@ void NestedTemplateArguments() {
   Outer<Inner<IndirectClass> >* opi;
   (void)opi;
 
+  // IWYU: IndirectClass needs a declaration
+  // IWYU: IndirectClass is...*indirect.h
+  FnUsingNested<IndirectClass>();
+
   // Test that use of template specialization type template argument is not
   // hidden by any sugar in the AST.
 
@@ -219,6 +236,28 @@ TemplateWithFwdDeclUse<TplInI1<IndirectClass>> c;
 
 // ---------------------------------------------------------------
 
+template <typename T>
+T TplParamRetType() {
+  throw 1;
+}
+
+// IWYU: IndirectClass needs a declaration
+IndirectClass& i = TplParamRetType<IndirectClass&>();
+
+// ---------------------------------------------------------------
+
+// Test that IWYU doesn't crash on function template type parameter packs.
+
+template <typename...>
+void TplFnWithParameterPack() {
+}
+
+void TestParameterPack() {
+  TplFnWithParameterPack();
+}
+
+// ---------------------------------------------------------------
+
 /**** IWYU_SUMMARY
 
 tests/cxx/template_args.cc should add these lines:
@@ -231,7 +270,7 @@ tests/cxx/template_args.cc should remove these lines:
 The full include-list for tests/cxx/template_args.cc:
 #include "tests/cxx/indirect.h"  // for IndirectClass
 #include "tests/cxx/template_args-d1.h"  // for ProvidingAlias
-#include "tests/cxx/template_args-d2.h"  // for NonProvidingAlias
+#include "tests/cxx/template_args-d2.h"  // for NonProvidingAlias, NonProvidingFunctionAlias1, NonProvidingFunctionAlias2
 #include "tests/cxx/template_args-i1.h"  // for TplInI1
 template <typename F> struct FunctionStruct;  // lines XX-XX
 
